#include "actor.h"
#include<QDebug>

#include "alarmtime.h"
#include "sensoractor.h"
#include "timeractor.h"
#include "regulator.h"
#include "polynomalactor.h"
#include "factoractor.h"

Actor::Actor(QObject *parent): Item(QRandomGenerator::global()->generate(), "", 0, parent)
{
}

Actor::~Actor()
{

}

void Actor::performAction()
{
    if(active)
    {
        sigValue(triggerValue);
    }
}

void Actor::makeActive()
{
    active = true;
}


void Actor::makeInactive()
{
    active = false;
}

QString Actor::actionName()
{
    QString string;
    if(triggerValue == 0 ) string = "off";
    else if(triggerValue == 1 )  string = "on";
    else string = "value to " + QString::number(triggerValue);
    return  string;
}

void Actor::setActive(uint8_t state)
{
    state ? makeActive() : makeInactive();
}

bool Actor::isActive()
{
    return active;
}

bool Actor::isExausted()
{
    return exausted;
}

void Actor::store(QJsonObject& json)
{
    Item::store(json);
    json["Active"] =  active;
    json["Exausted"] = exausted;
    json["TriggerValue"] = triggerValue;
}

void Actor::load(const QJsonObject& json, const bool preserve)
{
    Item::load(json, preserve);
    active = json["Active"].toBool();
    exausted = json["Exausted"].toBool();
    triggerValue = json["TriggerValue"].toInt();
}

void Actor::setTriggerValue(uint8_t value)
{
    triggerValue=value;
}

uint8_t Actor::getTriggerValue()
{
    return  triggerValue;
}

void Actor::onValueChanged(uint8_t value)
{

}

Actor* Actor::createActor(const QString& type)
{
    Actor* actor = nullptr;
    if(type == "Alarm") actor = new AlarmTime();
    else if(type == "Sensor") actor = new SensorActor();
    else if(type == "Timer") actor = new TimerActor();
    else if(type == "Regulator") actor = new Regulator();
    else if(type == "Polynomal") actor = new PolynomalActor();
    else if(type == "MultiFactor") actor = new MultiFactorActor();
    else if(type == "Actor") actor = new Actor();
    return actor;
}

Actor* Actor::loadActor(const QJsonObject &json)
{
    QString type = json["Type"].toString("Actor");
    Actor* actor = createActor(type);
    if(actor) actor->load(json);
    return actor;
}

void Actor::setValue(uint8_t value)
{
    Item::setValue(value);
    setActive(value);
}

