#include "itemstore.h"
#include "relay.h"
#include "messageitem.h"
#include <QJsonArray>

ItemStore::ItemStore(QObject *parent): QObject(parent)
{
}

void ItemStore::addItem(std::shared_ptr<Item> item)
{
    bool mached = false;
    for(unsigned i = 0; i < items_.size(); i++ ) if(*items_[i] == *item) mached = true;
    if(!mached)
    {
        items_.push_back(std::shared_ptr<Item>(item));
        itemAdded(std::weak_ptr<Item>(items_.back()));
    }
}

void ItemStore::addItems(const std::vector<std::shared_ptr<Item>>& itemIn)
{
    for(unsigned i = 0; i < items_.size(); i++ )
    {
        if(Relay* relay = dynamic_cast<Relay*>(items_[i].get()))
        {
            bool mached = false;
            for(unsigned j = 0; j < itemIn.size(); j++) if(*(items_[i]) == *(itemIn[j]))
            {
                mached = true;
                if(itemIn[j]->getValue() != items_[i]->getValue()) items_[i]->informValue(itemIn[j]->getValue());
                Relay* relayIn = dynamic_cast<Relay*>(itemIn[j].get());
                if(relayIn)
                {
                    if(relay->getId() != relayIn->getId()) relay->setId(relayIn->getId());
                }

            }
            if(!mached)
            {
                itemDeleted(*items_[i].get());
                items_.erase(items_.begin()+i);
            }
        }
    }

    for(unsigned j = 0; j < itemIn.size(); j++)addItem(itemIn[j]);

}

void ItemStore::removeItem(const ItemData& item)
{
    for(unsigned j = 0; j < items_.size(); j++)
    {
        if(item == *items_[j])
        {
            items_.erase(items_.begin()+j);
            --j;
        }
    }
}


void ItemStore::clear()
{
    for(size_t i = 0; i < items_.size(); ++i) itemDeleted(*items_[i]);
    items_.clear();
}


void ItemStore::itemStateChanged(const ItemData& item)
{
     for(unsigned i = 0; i < items_.size(); i++ )
     {
         if(items_[i]->operator==(item))
         {
            if(items_[i]->getValue() != item.getValue())items_[i]->informValue(item.getValue());
         }

     }

}

void ItemStore::store(QJsonObject& json)
{
    QJsonArray itemsArray;
    for(size_t i = 0; i < items_.size(); ++i)
    {
        QJsonObject itemObject;
        items_[i]->store(itemObject);
        itemsArray.append(itemObject);
    }
    json["Items"] = itemsArray;
}

void ItemStore::load(const QJsonObject& json)
{
    const QJsonArray itemsArray(json["Items"].toArray(QJsonArray()));
    for(int i = 0; i < itemsArray.size(); ++i)
    {
        if(itemsArray[i].isObject())
        {
            const QJsonObject itemObject = itemsArray[i].toObject();
            std::shared_ptr<Item> newItem;
            if(itemObject["Type"].toString("") == "Relay")
            {
                newItem = std::shared_ptr<Relay>(new Relay());
            }
            else if(itemObject["Type"].toString("") == "Message")
            {
                newItem = std::shared_ptr<MessageItem>(new MessageItem);
            }
            else if(itemObject["Type"].toString("") == "Aux")
            {
            }
            if(newItem)
            {
                newItem->load(itemObject);
                addItem(newItem);
            }
        }
    }
}
