#ifndef MICROCONTROLLER_H
#define MICROCONTROLLER_H

#include <iostream>

#include <QObject>
#include <QColor>
#include <QIODevice>
#include <QString>
#include <QRunnable>
#include <QDebug>
#include <QEventLoop>
#include <QTimer>
#include <QAbstractButton>
#include <vector>
#include <memory>
#include "items/item.h"
#include "items/relay.h"
#include "sensors/sensor.h"

class Microcontroller : public QObject
{

    Q_OBJECT

public:

    static constexpr char AMP_RELAY = 0;
    static constexpr char SENSOR_TEMPERATURE = 1;
    static constexpr char SENSOR_DOOR = 0 ;

private:

    bool listMode = false;

    //uint8_t _auxState = 0;

    QIODevice* _port = nullptr;

    std::vector< std::shared_ptr<Item> > relayList;

    QScopedPointer<QEventLoop> loop;
    QString _buffer;

    void processMicroReturn();
    void processList(const QString& buffer);
    void processRelayState(const QString& buffer);
    void processSensorState(const QString& buffer);
    std::shared_ptr<Relay> processRelayLine(const QString& buffer);

    void write(char *buffer, const size_t length);
    void write(const QByteArray& buffer);

public:
    Microcontroller(QIODevice* port);
    Microcontroller();
    ~Microcontroller();
    bool connected();
    void setIODevice(QIODevice* port);

public slots:
    void rgbOn();
    void rgbOff();
    void changeRgbColor(const QColor color);
    void setPattern(int pattern);
    void startSunrise();

    void requestState();

    void setAuxPwm(int duty);

    void relayOn(int relay);
    void relayOff(int relay);
    void relayToggle(int state, int id);

private slots:

    void isReadyRead();

signals:
    void textRecived(const QString string);
    void itemChanged(ItemData relay);
    void auxStateChanged(int value);
    void gotRelayList(std::vector< std::shared_ptr<Item> >&);

    void gotSensorState(Sensor sensor);
};

#endif // MICROCONTROLLER_H
