#include "actorsettingsdialog.h"
#include "ui_actorsettingsdialog.h"
#include "itemsettingsdialog.h"
#include <QDebug>
#include <QSpinBox>

ActorSettingsDialog::ActorSettingsDialog(AlarmTime* alarm, QWidget *parent):
    QDialog(parent),
    actor_(alarm),
    ui(new Ui::ActorSettingsDialog)
{
    init();

    widget = new AlarmWidget(alarm, this);
    ui->vertlayout->addWidget(widget);
}

ActorSettingsDialog::ActorSettingsDialog(SensorActor* actor, QWidget *parent) :
QDialog(parent),
actor_(actor),
ui(new Ui::ActorSettingsDialog)
{
    init();

    widget = new SensorActorWidget(actor, &globalSensors, this);
    ui->vertlayout->addWidget(widget);
}

ActorSettingsDialog::ActorSettingsDialog(Regulator* actor, QWidget *parent) :
    QDialog(parent),
    actor_(actor),
    ui(new Ui::ActorSettingsDialog)
{
    init();

    widget = new RegulatorWdiget(actor, &globalSensors, this);
    ui->vertlayout->addWidget(widget);
}

ActorSettingsDialog::ActorSettingsDialog(TimerActor* actor, QWidget *parent) :
QDialog(parent),
actor_(actor),
ui(new Ui::ActorSettingsDialog)
{
    init();

    widget = new TimerActorWidget(actor, this);
    ui->vertlayout->addWidget(widget);
}

ActorSettingsDialog::ActorSettingsDialog(PolynomalActor* actor, QWidget *parent) :
QDialog(parent),
actor_(actor),
ui(new Ui::ActorSettingsDialog)
{
    init();

    widget = new PolynomalActorWidget(actor, &globalSensors, this);
    ui->vertlayout->addWidget(widget);
}

ActorSettingsDialog::ActorSettingsDialog(MultiFactorActor* actor, QWidget *parent) :
QDialog(parent),
actor_(actor),
ui(new Ui::ActorSettingsDialog)
{
    init();
    widget = new FactorActorWidget(actor, this);
    ui->vertlayout->addWidget(widget);
}

ActorSettingsDialog::ActorSettingsDialog(Actor* actor, QWidget *parent) :
QDialog(parent),
actor_(actor),
ui(new Ui::ActorSettingsDialog)
{
    init();
}

void ActorSettingsDialog::init()
{
    ui->setupUi(this);
    connect(ui->comboBox_action, SIGNAL(currentIndexChanged(int)), this, SLOT(changeAction(int)));
    connect(ui->spinBox,  SIGNAL(valueChanged(int)), this, SLOT(valueChanged(int)));
    connect(ui->pushButton_editItem, &QPushButton::clicked, this, &ActorSettingsDialog::editAsItem);
    ui->spinBox->hide();

    ui->spinBox->setValue(actor_->getTriggerValue());
    if(actor_->getTriggerValue() == 0) ui->comboBox_action->setCurrentIndex(0);
    else if(actor_->getTriggerValue() == 1) ui->comboBox_action->setCurrentIndex(1);
    else ui->comboBox_action->setCurrentIndex(2);
}

ActorSettingsDialog::~ActorSettingsDialog()
{
    delete ui;
}

void ActorSettingsDialog::editAsItem()
{
    ItemSettingsDialog itemSettingsDiag(actor_, this);
    itemSettingsDiag.exec();
}

void ActorSettingsDialog::valueChanged(int value)
{
    actor_->setTriggerValue(value);
}

void ActorSettingsDialog::hideCancle(const bool hide)
{

}

void ActorSettingsDialog::changeAction(int index)
{
         if(index == 0) actor_->setTriggerValue(0);
    else if(index == 1) actor_->setTriggerValue(1);
    else if(index == 2)
    {
             ui->spinBox->show();
             actor_->setTriggerValue(ui->spinBox->value());
    }
    if(index != 2)ui->spinBox->hide();
}

