 #include "itemsettingsdialog.h"
#include "ui_itemsettingsdialog.h"
#include "actorsettingsdialog.h"
#include "../actors/alarmtime.h"
#include "../actors/sensoractor.h"
#include "../actors/timeractor.h"
#include "../actors/regulator.h"
#include "../actors/factoractor.h"

#include<memory>

ItemSettingsDialog::ItemSettingsDialog(Item* item, QWidget *parent) :
    QDialog(parent),
    item_(item),
    ui(new Ui::ItemSettingsDialog)
{
    ui->setupUi(this);

    setModal(false);

    ui->label_name->setText(item_->getName());

    if(Relay* relay = dynamic_cast<Relay*>(item_))
    {
        ui->label_address->setText(QString::number(relay->getAddress(), 2));
        ui->label_id->setText(QString::number(relay->getId(), 10));
    }
    else
    {
        ui->label_address->hide();
        ui->label_id->hide();
        ui->label_address_lable->hide();
        ui->label_id_lable->hide();
    }

    connect(ui->pushButton_add, &QPushButton::clicked, this, &ItemSettingsDialog::addActor);
    connect(ui->pushButton_remove, &QPushButton::clicked, this, &ItemSettingsDialog::removeActor);
    connect(ui->pushButton_edit, &QPushButton::clicked, this, &ItemSettingsDialog::editActor);

     ui->tableWidget->setHorizontalHeaderItem(0, new QTableWidgetItem("Actor"));
     ui->tableWidget->setHorizontalHeaderItem(1, new QTableWidgetItem("Action"));
     ui->tableWidget->setHorizontalHeaderItem(2, new QTableWidgetItem("Enabled"));
     ui->tableWidget->horizontalHeader()->setSectionResizeMode(0, QHeaderView::Stretch);
     ui->tableWidget->horizontalHeader()->resizeSection(1, 60);
     ui->tableWidget->horizontalHeader()->resizeSection(2, 75);
    loadActorList();
}

ItemSettingsDialog::~ItemSettingsDialog()
{
    delete ui;
}

void ItemSettingsDialog::loadActorList()
{
    //ui->listWidget->clear();
    ui->tableWidget->setRowCount(item_->getActors().size());

    for(unsigned i = 0; i < item_->getActors().size(); i++)
    {
        ui->tableWidget->setItem(i, 0, new QTableWidgetItem(item_->getActors()[i]->getName()));
        ui->tableWidget->setItem(i, 1, new QTableWidgetItem(item_->getActors()[i]->actionName()));
        ui->tableWidget->setItem(i, 2, new QTableWidgetItem(item_->getActors()[i]->isActive() ? "Y" : "N"));
    }
}

void ItemSettingsDialog::addActor()
{
    ActorSettingsDialog* dialog = nullptr;
    Actor* actor = nullptr;

    if(ui->comboBox->currentText() == "Alarm")
    {
        AlarmTime* alarm = new AlarmTime;
        actor = alarm;
        dialog = new ActorSettingsDialog(alarm, this);
    }
    else if(ui->comboBox->currentText() == "Sensor")
    {
        SensorActor* sensorActor = new SensorActor();
        actor = sensorActor;
        dialog = new ActorSettingsDialog(sensorActor, this);
    }
    else if(ui->comboBox->currentText() == "Timer" )
    {
        TimerActor* timerActor = new TimerActor();
        actor = timerActor;
        dialog = new ActorSettingsDialog(timerActor, this);
    }
    else if(ui->comboBox->currentText() == "Regulator")
    {
        Regulator* regulator = new Regulator();
        actor = regulator;
        dialog = new ActorSettingsDialog(regulator, this);
    }

    else if(ui->comboBox->currentText() == "Polynomal")
    {
        PolynomalActor* polynomalActor = new PolynomalActor();
        actor = polynomalActor;
        dialog = new ActorSettingsDialog(polynomalActor, this);
    }

    else if(ui->comboBox->currentText() == "Multi Factor")
    {
        MultiFactorActor* polynomalActor = new MultiFactorActor();
        actor = polynomalActor;
        dialog = new ActorSettingsDialog(polynomalActor, this);
    }


    if(dialog != nullptr)
    {
        dialog->setParent(this);
        dialog->show();
        if(dialog->exec() == QDialog::Accepted)
        {
            item_->addActor(actor);
            loadActorList();
        }
        else delete actor;
        delete dialog;
    }

}

void ItemSettingsDialog::removeActor()
{
    if(item_->getActors().size() > ui->tableWidget->currentRow())
    {
       item_->removeActor(item_->getActors().at(ui->tableWidget->currentRow()));
       loadActorList();
    }
}

void ItemSettingsDialog::editActor()
{
    if(item_->getActors().size() > ui->tableWidget->currentRow())
    {
        Actor* actor = item_->getActors()[ui->tableWidget->currentRow()];

        AlarmTime* alarmTime = dynamic_cast<AlarmTime*>(actor);
        Regulator* regulator = dynamic_cast<Regulator*>(actor);
        SensorActor* sensorActor = dynamic_cast<SensorActor*>(actor);
        TimerActor* timerActor = dynamic_cast<TimerActor*>(actor);
        PolynomalActor* polynomalActor = dynamic_cast<PolynomalActor*>(actor);
        MultiFactorActor* factorActor = dynamic_cast<MultiFactorActor*>(actor);

        ActorSettingsDialog* dialog;

        if(alarmTime) dialog = new ActorSettingsDialog(alarmTime, this);
        else if(regulator)  dialog = new ActorSettingsDialog(regulator, this);
        else if(sensorActor) dialog = new ActorSettingsDialog(sensorActor, this);
        else if(timerActor) dialog = new ActorSettingsDialog(timerActor, this);
        else if(polynomalActor) dialog = new ActorSettingsDialog(polynomalActor, this);
        else if(factorActor) dialog = new ActorSettingsDialog(factorActor, this);
        else dialog = new ActorSettingsDialog(actor, this);
        dialog->setParent(this);
        dialog->show();
        dialog->exec();

        for(int i = 0; i < ui->tableWidget->rowCount() && i < item_->getActors().size(); ++i)
        {
            ui->tableWidget->item(i, 0)->setText(item_->getActors()[i]->getName());
            ui->tableWidget->item(i, 1)->setText(item_->getActors()[i]->actionName());
            ui->tableWidget->item(i, 2)->setText(item_->getActors()[i]->isActive() ? "Y" : "N");
        }
    }
}
